#define F_CPU 4000000UL 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define X_MIN			3500	// <
#define X_MIN_CENTER	4500
#define X_CENTER		5500
#define X_MAX_CENTER	6500
#define X_MAX			7500	// >

#define Y_MIN			3900	// v
#define Y_MIN_CENTER	4900
#define Y_CENTER		5900
#define Y_MAX_CENTER	6900
#define Y_MAX			7900	// ^

uint8_t servo=4;

unsigned volatile long int x_pos=X_CENTER;
unsigned volatile long int y_pos=Y_CENTER;

void PWM_Init(void);
void Output_Init(void);
void Switch_Init(void);

int main(void)
{
	sei();
	PWM_Init();
	Output_Init();
	
	x_pos=X_CENTER;
	y_pos=Y_CENTER;
	
	while(1)
	{
		if(!(PINC & (1 << 4)))
		{
			if(x_pos<X_MAX)
			{
				x_pos++;
				_delay_us(50);
			}
		}
		if(!(PINC & (1 << 5)))
		{
			if(x_pos>X_MIN)
			{
				x_pos--;
				_delay_us(50);				
			}
		}
		if(!(PINC & (1 << 6)))
		{
			if(y_pos<Y_MAX)
			{
				y_pos++;
				_delay_us(50);				
			}
		}
		if(!(PINC & (1 << 7)))
		{
			if(y_pos>Y_MIN)
			{
				y_pos--;
				_delay_us(50);				
			}
		}		
	}
	return 0;
}

void PWM_Init(void)
{
	// Fast PWM (TOP value is ICR1)
	// Clear OC1A/OC1B on compare match
	// Prescaler 1
	TCCR1A |= _BV(WGM11) | _BV(COM1A1);
	TCCR1B |= _BV(WGM12) | _BV(WGM13) | _BV(CS10);
	//Output Compare and Overflow 1B Interrupt Enable
	TIMSK |= (1<<OCIE1A) | (1<<TOIE1);	
	// TOP = (fCLK / (fPWM * 2 * N)) - 1
	ICR1 = 39999; //50Hz
}

void Output_Init(void)
{
	DDRA |= (1 << 4);	// Servo #2 Y axis
	DDRA |= (1 << 5);	// Servo #1 X axis
}

void Switch_Init(void)
{
	DDRC &= ~(1 << 4);
	DDRC &= ~(1 << 5);
	DDRC &= ~(1 << 6);
	DDRC &= ~(1 << 7);	
	PORTC |= (1 << 4);
	PORTC |= (1 << 5);
	PORTC |= (1 << 6);
	PORTC |= (1 << 7);		
}

ISR(TIMER1_COMPA_vect)
{
	PORTA &= ~(1 << servo);

	if(servo==4)
	{
		/////// Servo #1 X axis 
		OCR1A = x_pos;
		servo=5;
	}
	else
	{
		/////// Servo #2 Y axis
		OCR1A = y_pos;
		servo=4;
	}
}

ISR(TIMER1_OVF_vect)
{
	PORTA |= (1 << servo);	
}
